From 2965a981b7ec22866fbdf2d567d87e2d068d3617 Mon Sep 17 00:00:00 2001
From: Thomas Bruederli <thomas@roundcube.net>
Date: Fri, 31 Jul 2015 16:04:08 -0400
Subject: [PATCH] Allow to search and import missing PGP pubkeys from keyservers using Publickey.js
---
program/lib/Roundcube/rcube_mime.php | 147 +++++++++++++++++++++++++++++++------------------
1 files changed, 93 insertions(+), 54 deletions(-)
diff --git a/program/lib/Roundcube/rcube_mime.php b/program/lib/Roundcube/rcube_mime.php
index 572540f..e9cab61 100644
--- a/program/lib/Roundcube/rcube_mime.php
+++ b/program/lib/Roundcube/rcube_mime.php
@@ -3,8 +3,8 @@
/*
+-----------------------------------------------------------------------+
| This file is part of the Roundcube Webmail client |
- | Copyright (C) 2005-2012, The Roundcube Dev Team |
- | Copyright (C) 2011-2012, Kolab Systems AG |
+ | Copyright (C) 2005-2014, The Roundcube Dev Team |
+ | Copyright (C) 2011-2014, Kolab Systems AG |
| |
| Licensed under the GNU General Public License version 3 or |
| any later version with exceptions for skins & plugins. |
@@ -39,7 +39,6 @@
self::$default_charset = $default_charset;
}
-
/**
* Returns message/object character set name
*
@@ -58,7 +57,6 @@
return RCUBE_CHARSET;
}
-
/**
* Parse the given raw message source and return a structure
* of rcube_message_part objects.
@@ -75,7 +73,6 @@
return self::structure_part($struct);
}
-
/**
* Recursive method to convert a Mail_mimeDecode part into a rcube_message_part object
*
@@ -88,28 +85,30 @@
private static function structure_part($part, $count=0, $parent='')
{
$struct = new rcube_message_part;
- $struct->mime_id = $part->mime_id ? $part->mime_id : (empty($parent) ? (string)$count : "$parent.$count");
- $struct->headers = $part->headers;
- $struct->ctype_primary = $part->ctype_primary;
- $struct->ctype_secondary = $part->ctype_secondary;
- $struct->mimetype = $part->ctype_primary . '/' . $part->ctype_secondary;
+ $struct->mime_id = $part->mime_id ?: (empty($parent) ? (string)$count : "$parent.$count");
+ $struct->headers = $part->headers;
+ $struct->mimetype = $part->ctype_primary . '/' . $part->ctype_secondary;
+ $struct->ctype_primary = $part->ctype_primary;
+ $struct->ctype_secondary = $part->ctype_secondary;
$struct->ctype_parameters = $part->ctype_parameters;
- if ($part->headers['content-transfer-encoding'])
+ if ($part->headers['content-transfer-encoding']) {
$struct->encoding = $part->headers['content-transfer-encoding'];
- if ($part->ctype_parameters['charset'])
- $struct->charset = $part->ctype_parameters['charset'];
+ }
- $part_charset = $struct->charset ? $struct->charset : self::get_charset();
+ if ($part->ctype_parameters['charset']) {
+ $struct->charset = $part->ctype_parameters['charset'];
+ }
+
+ $part_charset = $struct->charset ?: self::get_charset();
// determine filename
if (($filename = $part->d_parameters['filename']) || ($filename = $part->ctype_parameters['name'])) {
$struct->filename = rcube_mime::decode_mime_string($filename, $part_charset);
}
- // copy part body and convert it to UTF-8 if necessary
- $struct->body = $part->ctype_primary == 'text' || !$part->ctype_parameters['charset'] ? rcube_charset::convert($part->body, $part_charset) : $part->body;
- $struct->size = strlen($part->body);
+ $struct->body = $part->body;
+ $struct->size = strlen($part->body);
$struct->disposition = $part->disposition;
foreach ((array)$part->parts as $child_part) {
@@ -118,7 +117,6 @@
return $struct;
}
-
/**
* Split an address list into a structured array list
@@ -169,7 +167,6 @@
return $out;
}
-
/**
* Decode a message header value
*
@@ -184,7 +181,6 @@
return $str;
}
-
/**
* Decode a mime-encoded string to internal charset
@@ -282,12 +278,12 @@
return rcube_charset::convert($input, $default_charset);
}
-
/**
* Decode a mime part
*
* @param string $input Input string
* @param string $encoding Part encoding
+ *
* @return string Decoded string
*/
public static function decode($input, $encoding = '7bit')
@@ -308,10 +304,8 @@
}
}
-
/**
* Split RFC822 header string into an associative array
- * @access private
*/
public static function parse_headers($headers)
{
@@ -331,7 +325,6 @@
return $a_headers;
}
-
/**
* @access private
@@ -366,6 +359,9 @@
$address = 'MAILER-DAEMON';
$name = substr($val, 0, -strlen($m[1]));
}
+ else if (preg_match('/('.$email_rx.')/', $val, $m)) {
+ $name = $m[1];
+ }
else {
$name = $val;
}
@@ -378,21 +374,26 @@
}
if ($decode) {
$name = self::decode_header($name, $fallback);
+ // some clients encode addressee name with quotes around it
+ if ($name[0] == '"' && $name[strlen($name)-1] == '"') {
+ $name = substr($name, 1, -1);
+ }
}
}
if (!$address && $name) {
$address = $name;
+ $name = '';
}
if ($address) {
+ $address = self::fix_email($address);
$result[$key] = array('name' => $name, 'address' => $address);
}
}
return $result;
}
-
/**
* Explodes header (e.g. address-list) string into array of strings
@@ -468,19 +469,20 @@
return $result;
}
-
/**
* Interpret a format=flowed message body according to RFC 2646
*
* @param string $text Raw body formatted as flowed text
*
* @return string Interpreted text with unwrapped lines and stuffed space removed
*/
- public static function unfold_flowed($text)
+ public static function unfold_flowed($text, $mark = null)
{
- $text = preg_split('/\r?\n/', $text);
- $last = -1;
+ $text = preg_split('/\r?\n/', $text);
+ $last = -1;
$q_level = 0;
+ $marks = array();
foreach ($text as $idx => $line) {
if (preg_match('/^(>+)/', $line, $m)) {
@@ -500,6 +502,10 @@
) {
$text[$last] .= $line;
unset($text[$idx]);
+
+ if ($mark) {
+ $marks[$last] = true;
+ }
}
else {
$last = $idx;
@@ -512,14 +518,18 @@
}
else {
// remove space-stuffing
- $line = preg_replace('/^\s/', '', $line);
+ $line = preg_replace('/^ /', '', $line);
- if (isset($text[$last]) && $line
+ if (isset($text[$last]) && $line && !$q_level
&& $text[$last] != '-- '
&& $text[$last][strlen($text[$last])-1] == ' '
) {
$text[$last] .= $line;
unset($text[$idx]);
+
+ if ($mark) {
+ $marks[$last] = true;
+ }
}
else {
$text[$idx] = $line;
@@ -530,15 +540,20 @@
$q_level = $q;
}
+ if (!empty($marks)) {
+ foreach (array_keys($marks) as $mk) {
+ $text[$mk] = $mark . $text[$mk];
+ }
+ }
+
return implode("\r\n", $text);
}
-
/**
* Wrap the given text to comply with RFC 2646
*
- * @param string $text Text to wrap
- * @param int $length Length
+ * @param string $text Text to wrap
+ * @param int $length Length
* @param string $charset Character encoding of $text
*
* @return string Wrapped text
@@ -570,7 +585,6 @@
return implode("\r\n", $text);
}
-
/**
* Improved wordwrap function with multibyte support.
@@ -637,7 +651,8 @@
if ($nextChar === ' ' || $nextChar === $separator) {
$afterNextChar = mb_substr($string, $width + 1, 1);
- if ($afterNextChar === false) {
+ // Note: mb_substr() does never return False
+ if ($afterNextChar === false || $afterNextChar === '') {
$subString .= $nextChar;
}
@@ -650,16 +665,16 @@
$subString = mb_substr($subString, 0, $spacePos);
$cutLength = $spacePos + 1;
}
- else if ($cut === false && $breakPos === false) {
- $subString = $string;
- $cutLength = null;
- }
else if ($cut === false) {
$spacePos = mb_strpos($string, ' ', 0);
- if ($spacePos !== false && $spacePos < $breakPos) {
+ if ($spacePos !== false && ($breakPos === false || $spacePos < $breakPos)) {
$subString = mb_substr($string, 0, $spacePos);
$cutLength = $spacePos + 1;
+ }
+ else if ($breakPos === false) {
+ $subString = $string;
+ $cutLength = null;
}
else {
$subString = mb_substr($string, 0, $breakPos);
@@ -667,7 +682,6 @@
}
}
else {
- $subString = mb_substr($subString, 0, $width);
$cutLength = $width;
}
}
@@ -691,7 +705,6 @@
return implode($break, $result);
}
-
/**
* A method to guess the mime_type of an attachment.
*
@@ -708,12 +721,20 @@
*/
public static function file_content_type($path, $name, $failover = 'application/octet-stream', $is_stream = false, $skip_suffix = false)
{
- $mime_type = null;
- $mime_magic = rcube::get_instance()->config->get('mime_magic');
- $mime_ext = $skip_suffix ? null : @include(RCUBE_CONFIG_DIR . '/mimetypes.php');
+ static $mime_ext = array();
+
+ $mime_type = null;
+ $config = rcube::get_instance()->config;
+ $mime_magic = $config->get('mime_magic');
+
+ if (!$skip_suffix && empty($mime_ext)) {
+ foreach ($config->resolve_paths('mimetypes.php') as $fpath) {
+ $mime_ext = array_merge($mime_ext, (array) @include($fpath));
+ }
+ }
// use file name suffix with hard-coded mime-type map
- if (is_array($mime_ext) && $name) {
+ if (!$skip_suffix && is_array($mime_ext) && $name) {
if ($suffix = substr($name, strrpos($name, '.')+1)) {
$mime_type = $mime_ext[strtolower($suffix)];
}
@@ -757,12 +778,13 @@
return $mime_type;
}
-
/**
* Get mimetype => file extension mapping
*
- * @param string Mime-Type to get extensions for
- * @return array List of extensions matching the given mimetype or a hash array with ext -> mimetype mappings if $mimetype is not given
+ * @param string Mime-Type to get extensions for
+ *
+ * @return array List of extensions matching the given mimetype or a hash array
+ * with ext -> mimetype mappings if $mimetype is not given
*/
public static function get_mime_extensions($mimetype = null)
{
@@ -790,6 +812,7 @@
$file_paths[] = '/etc/httpd2/mime.types';
$file_paths[] = '/etc/apache/mime.types';
$file_paths[] = '/etc/apache2/mime.types';
+ $file_paths[] = '/etc/nginx/mime.types';
$file_paths[] = '/usr/local/etc/httpd/conf/mime.types';
$file_paths[] = '/usr/local/etc/apache/conf/mime.types';
}
@@ -802,7 +825,7 @@
}
$mime_types = $mime_extensions = array();
- $regex = "/([\w\+\-\.\/]+)\t+([\w\s]+)/i";
+ $regex = "/([\w\+\-\.\/]+)\s+([\w\s]+)/i";
foreach((array)$lines as $line) {
// skip comments or mime types w/o any extensions
if ($line[0] == '#' || !preg_match($regex, $line, $matches))
@@ -818,7 +841,9 @@
// fallback to some well-known types most important for daily emails
if (empty($mime_types)) {
- $mime_extensions = (array) @include(RCUBE_CONFIG_DIR . '/mimetypes.php');
+ foreach (rcube::get_instance()->config->resolve_paths('mimetypes.php') as $fpath) {
+ $mime_extensions = array_merge($mime_extensions, (array) @include($fpath));
+ }
foreach ($mime_extensions as $ext => $mime) {
$mime_types[$mime][] = $ext;
@@ -852,7 +877,6 @@
return $mimetype ? $mime_types[$mimetype] : $mime_extensions;
}
-
/**
* Detect image type of the given binary data by checking magic numbers.
*
@@ -871,4 +895,19 @@
return 'image/' . $type;
}
+ /**
+ * Try to fix invalid email addresses
+ */
+ public static function fix_email($email)
+ {
+ $parts = rcube_utils::explode_quoted_string('@', $email);
+ foreach ($parts as $idx => $part) {
+ // remove redundant quoting (#1490040)
+ if ($part[0] == '"' && preg_match('/^"([a-zA-Z0-9._+=-]+)"$/', $part, $m)) {
+ $parts[$idx] = $m[1];
+ }
+ }
+
+ return implode('@', $parts);
+ }
}
@@ -468,19 +469,20 @@
* Interpret a format=flowed message body according to RFC 2646
*
- * @param string $text Raw body formatted as flowed text
+ * @param string $text Raw body formatted as flowed text
+ * @param string $mark Mark each flowed line with specified character
*
* @return string Interpreted text with unwrapped lines and stuffed space removed
--
Gitblit v1.9.1